#include "ModificationVisitor.h"

using namespace osgTerrain;

ModificationVisitor::ModificationVisitor(std::string extensionToSet) : _extensionToSet(extensionToSet)
{
	setTraversalMode( osg::NodeVisitor::TRAVERSE_ALL_CHILDREN );
	_technique = new rampedEllipsoidTechnique();

	// Set ROI to flatten and estimated height after correction
	_technique->setModifiedHeight(600);	// 450
}

void ModificationVisitor::apply( osg::Node& node )
{
	//OSG_NOTIFY( osg::ALWAYS ) << "ModVisitor: Node found: " << node.className() << std::endl;

	if(std::string(node.className())=="TerrainTile")
	{
		//OSG_NOTIFY( osg::ALWAYS ) << "ModVisitor: Tile found" << std::endl;
		osgTerrain::TerrainTile* tile = dynamic_cast<osgTerrain::TerrainTile*>(&node);
		if(tile)
			modifyTile(tile);
	}

	traverse( node );
}

void ModificationVisitor::apply( osg::PagedLOD& pNode )
{
	//OSG_NOTIFY( osg::ALWAYS ) << "ModVisitor: pagedLOD found: " << pNode.className() << std::endl;

	for( unsigned int i=0;i<pNode.getNumFileNames();i++)
	{
		//OSG_NOTIFY( osg::ALWAYS ) << "ModVisitor: pagedLOD Child #" << i << std::endl;
		if(pNode.getFileName(i)!="")
		{
			//OSG_NOTIFY( osg::ALWAYS ) << "ModVisitor: pagedLOD Child Name pre: " << pNode.getFileName(i) << std::endl;
			pNode.setFileName(i, pNode.getFileName(i)+_extensionToSet );
			//OSG_NOTIFY( osg::ALWAYS ) << "ModVisitor: pagedLOD Child Name post: " << newFileName << std::endl;
		}
	}

	traverse( pNode );
}

void ModificationVisitor::modifyTile(osgTerrain::TerrainTile* tile)
{
	//OSG_NOTIFY( osg::ALWAYS ) << "ModificationVisitor::modifyTile() : Checking if tile is affected.." << std::endl;
	HeightFieldLayer* hfl = dynamic_cast<HeightFieldLayer*>(tile->getElevationLayer());
	osg::HeightField* h = hfl->getHeightField();

	// ROI to flatten
	//osg::Vec4d modificationROI = osg::Vec4d( 48.336808, 48.370467, 11.736750, 11.835322 );		// roi_lat_min, roi_lat_max, roi_lon_min, roi_lon_max
	osg::Vec4d modificationROI = osg::Vec4d( 48.331808, 48.375467, 11.731750, 11.840322 );		// roi_lat_min, roi_lat_max, roi_lon_min, roi_lon_max

	// Determine extend of the tile.
	double lat_min = h->getOrigin()[1];
	double lat_max = lat_min + h->getNumRows() * h->getYInterval();
	double lon_min = h->getOrigin()[0];
	double lon_max = lon_min + h->getNumColumns() * h->getXInterval();

	// Check if tile is fully or partially inside ROI:
	if(lat_max>modificationROI[0] && lat_min<modificationROI[1]			// lat inside ROI?
		&& lon_max>modificationROI[2] && lon_min<modificationROI[3] )	// lon inside ROI?
	{
		_technique->modifyHeightfield( modificationROI, h, osg::Vec4d(lat_min, lat_max, lon_min, lon_max) );
	}
}